#include "StdAfx.h"
#include "IniStream.h"

IniStream::IniStream(String ^fileName)
{
	m_fileName=fileName;
	m_modified=false;

	m_data=gcnew IniFileDictionary();
	
	Open();
}

#pragma warning(disable:4706)
void IniStream::Open(void)
{
	if(!File::Exists(m_fileName))
		return;

	StreamReader ^sr=gcnew StreamReader(m_fileName);
	String ^line,^key,^subkey,^value;
	int pos;
	bool nuevaKey=false;
	
	while(line=sr->ReadLine())
	{

		//Es una seccin, activamos la cadena para el siguiente valor
		if(line->StartsWith("[") && line->EndsWith("]"))
		{
			key=line->Substring(1,line->Length-2);
			nuevaKey=true;
			continue;
		}

		//Componemos los pares
		pos=line->LastIndexOf('=');
		if(pos!=-1)	//Si NO es un comentario (o sea, tiene =)
		{
			subkey=line->Substring(0,pos);
			value=line->Substring(pos+1);
		}
		else
		{
			subkey=line;
			value=nullptr;
			if(key==nullptr)	//Es un comentario al principio de todo, forzamos una key vaca.
			{
				nuevaKey=true;			
				key="";
			}
		}
		
		//Si es una seccin nueva...
		if(nuevaKey)
		{
			Dictionary<String ^,String ^>^dict=gcnew Dictionary<String ^,String ^>();
			dict->Add(subkey,value);
			m_data->Add(key,dict);
			nuevaKey=false;
		}
		else
			m_data[key]->Add(subkey,value);
	}
	sr->Close();
}
#pragma warning(default:4706)
void IniStream::Open(String ^fileName)
{
	Close();
	m_fileName=fileName;
	Open();
}
void IniStream::Close(void)
{
	if(m_modified)
	{
		StreamWriter ^stream=gcnew StreamWriter(m_fileName,false);
		for each(KeyValuePair<String ^,Dictionary<String ^,String ^>^>^dict in m_data)
		{
			stream->WriteLine("["+dict->Key+"]");
			for each(KeyValuePair<String ^,String ^>^item in dict->Value)
			{
				if(item->Value!=nullptr)		
					stream->WriteLine(item->Key+"="+item->Value);
				else
					stream->WriteLine(item->Key);
			}
		}
		stream->Close();

		m_modified=false;
		m_data->Clear();
	}
}
#pragma region Lectura/Escritura de cadenas
String ^IniStream::ReadString(String ^section,String ^name,String ^def)
{
	try{
	String ^s=m_data[section][name];
	return s;
	} catch (KeyNotFoundException ^){
		return def;
	}
}
String ^IniStream::ReadString(System::String ^name, System::String ^def)
{
	return ReadString(m_currentSection,name,def);
}
String ^IniStream::ReadString(String ^name)
{
	return ReadString(m_currentSection,name,"");
}
void IniStream::WriteString(System::String ^section, System::String ^name,String ^value)
{
	if(m_data->ContainsKey(section))
	{
		if(m_data[section]->ContainsKey(name))
		{
			String ^temp=m_data[section][name];
			if(temp!=value)
			{
				m_data[section][name]=value;
				m_modified=true;
			}
		}
		else
		{
			m_data[section]->Add(name,value);
			m_modified=true;
		}
	}
	else
	{
		Dictionary<String ^,String ^>^dict=gcnew Dictionary<String ^,String ^>();
		dict->Add(name,value);
		m_data->Add(section,dict);
		m_modified=true;
	}
}
void IniStream::WriteString(System::String ^name, System::String ^value)
{
	WriteString(m_currentSection,name,value);
}
#pragma endregion
#pragma region Lectura/Escritura de enteros (Int32)
Int32 IniStream::ReadInt32(String ^section,String ^name,Int32 def)
{
	String ^temp=ReadString(section,name,def.ToString());
	Int32 value;
	try {
		value=Convert::ToInt16(temp);
	} catch (Exception ^) {
		value=def;
	}
	return value;
}
Int32 IniStream::ReadInt32(String ^name,Int32 def)
{
	return ReadInt32(m_currentSection,name,def);
}
void IniStream::WriteInt32(String ^section,String ^name,Int32 value)
{
	WriteString(section,name,value.ToString());
}
void IniStream::WriteInt32(String ^name,Int32 value)
{
	WriteString(m_currentSection,name,value.ToString());
}
#pragma endregion
#pragma region Lectura/Escritura de bool
bool IniStream::ReadBool(String ^section,String ^name,bool def)
{
	String ^s=ReadString(section,name,def.ToString());
	bool res;
	try{ 
		res=Convert::ToBoolean(s);
	}catch(Exception ^){
		res=def;
	}
	return res;
}
bool IniStream::ReadBool(String ^name,bool def)
{
	return ReadBool(m_currentSection,name,def);
}
void IniStream::WriteBool(String ^section,String ^name,bool value)
{
	WriteString(section,name,value.ToString());
}
void IniStream::WriteBool(String ^name,bool value)
{
	WriteString(m_currentSection,name,value.ToString());
}
#pragma endregion
